@@ -10,7 +10,7 @@ from requests.auth import HTTPBasicAuth
 
 
 #-----------------------------------------------------------------------------
-__all__ = 'Job', 'Jenkins', 'Server', 'JenkinsError', 'Build'
+__all__ = 'Job', 'Jenkins', 'Server', 'JenkinsError', 'Build', 'View'
 
 
 #-----------------------------------------------------------------------------
@@ -117,6 +117,18 @@ class Job(object):
     def config(self, newconfig):
         self.reconfigure(newconfig)
 
+    @property
+    def config_etree(self):
+        # The cost of `'lxml' in sys.modules' is negligible and is
+        # preferable to having a hard dependency on lxml.
+        from lxml import etree
+        return etree.fromstring(self.config)
+
+    @config_etree.setter
+    def config_etree(self, newconfig):
+        newconfig = newconfig.tostring(etree)
+        self.reconfigure(newconfig)
+
     @property
     def builds(self):
         return [Build(self, i['number']) for i in self.info['builds']]
@@ -235,6 +247,32 @@ class View(object):
             raise JenkinsError(msg % self.name)
         return res.text
 
+    @config.setter
+    def config(self, newconfig):
+        self.reconfigure(newconfig)
+
+    @property
+    def config_etree(self):
+        # The cost of `'lxml' in sys.modules' is negligible and is
+        # preferable to having a hard dependency on lxml.
+        from lxml import etree
+        return etree.fromstring(self.config)
+
+    @config_etree.setter
+    def config_etree(self, newconfig):
+        newconfig = newconfig.tostring(etree)
+        self.reconfigure(newconfig)
+
+    def delete(self):
+        '''Permanently remove view.'''
+        self._not_exist_raise()
+        url = 'view/%s/doDelete' % quote(self.name)
+
+        res = self.server.post(url, throw=False)
+        if self.exists:
+            raise JenkinsError('delete of view "%s" failed' % self.name)
+        return res
+
     def reconfigure(self, newconfig):
         '''Update the config.xml of an existing view.'''
         self._not_exist_raise()
@@ -278,6 +316,23 @@ class View(object):
             msg = 'could not add job "%s" to view "%s"'
             raise JenkinsError(msg % (job.name, self.name))
 
+    @classmethod
+    def create(cls, name, configxml, server):
+        '''Create a new Jenkins view.'''
+
+        view = cls(name, server)
+        if view.exists:
+            raise JenkinsError('view "%s" already exists' % name)
+
+        headers = {'Content-Type': 'text/xml'}
+        params = {'name': name}
+        res = server.post('createView', data=configxml, params=params, headers=headers, throw=False)
+
+        if not res or res.status_code != 200:
+            raise JenkinsError('create "%s" failed' % name)
+        else:
+            res.raise_for_status()
+
 
 #-----------------------------------------------------------------------------
 class Build(object):
@@ -417,6 +472,14 @@ class Jenkins(object):
         job.config = newconfig
         return job
 
+    def job_config_etree(self, name):
+        return self.job(name).config_etree
+
+    def job_reconfigure_etree(self, name, newconfig):
+        job = self.job(name)
+        job.config_etree = newconfig
+        return job
+
     def job_build(self, name, parameters=None, token=None):
         return self.job(name).build(parameters, token)
 
@@ -445,6 +508,22 @@ class Jenkins(object):
     def view_config(self, name):
         return self.view(name).config
 
+    def view_reconfigure(self, name, newconfig):
+        view = self.view(name)
+        view.config = newconfig
+        return view
+
+    def view_config_etree(self, name):
+        return self.view(name).config_etree
+
+    def view_delete(self, name):
+        return self.view(name).delete()
+
+    def view_reconfigure_etree(self, name, newconfig):
+        view = self.view(name)
+        view.config_etree = newconfig
+        return view
+
     def view_add_job(self, name, job_name):
         job = self.job(job_name)
         return self.view(name).add_job(job)
@@ -453,6 +532,9 @@ class Jenkins(object):
         job = self.job(job_name)
         return self.view(name).remove_job(job)
 
+    def view_create(self, name, config):
+        return View.create(name, config, self.server)
+
     job_exists.__doc__ = Job.exists.__doc__
     job_delete.__doc__ = Job.delete.__doc__
     job_enable.__doc__ = Job.enable.__doc__
@@ -465,6 +547,8 @@ class Jenkins(object):
     view_exists.__doc__ = View.exists.__doc__
     view_add_job.__doc__ = View.add_job.__doc__
     view_remove_job.__doc__ = View.remove_job.__doc__
+    view_create.__doc__ = View.create.__doc__
+    view_delete.__doc__ = View.delete.__doc__
 
 
 #-----------------------------------------------------------------------------
@@ -477,7 +561,7 @@ class JenkinsError(Exception):
 
 
 #-----------------------------------------------------------------------------
-## uncomment to enable http logging
+# uncomment to enable http logging
 # try:
 #     import logging, httplib
 #     httplib.HTTPConnection.debuglevel = 1
